#include <field3d.hxx>
#include <globals.hxx>
#include <derivs.hxx>
#include <difops.hxx>
#include <bout/mesh.hxx>
#include <invert_laplace.hxx>

void c_set_f3d_all(Field3D * f, const double * data){
  int j=0;
  f->allocate();
  for (auto i : (*f)){
    (*f)[i]=data[j++];
  }
}

void c_set_f3d_all_(Field3D * f, const double data){
  (*f)=data;
}

void c_get_f3d_all(const Field3D * f, double * data){
  int j=0;
  for (auto i : (*f)){
    data[j++]=(*f)[i];
  }
}

int getNx( Field3D * a){
  return a->getNx();
}

int getNy( Field3D * a){
  return a->getNy();
}

int getNz( Field3D * a){
  return a->getNz();
}

Field3D * fadd(Field3D*a,Field3D*b){
  Field3D * r=new Field3D(*a);
  *r += *b;
  return r;
}

Field3D * fadd(Field3D*a,double b){
  Field3D * r=new Field3D(*a);
  *r += b;
  return r;
}

Field3D * fadd(double b,Field3D*a){
  return new Field3D(b + *a);
}

void fiadd(Field3D*a,Field3D*b){
  *a += *b;
}

void fiadd(Field3D*a,double b){
  *a += b;
}

Field3D * fadd(Field3D*a,Field2D*b){
  Field3D * r=new Field3D(*a);
  *r += *b;
  return r;
}

Field3D * fadd(Field2D*a,Field3D*b){
  return new Field3D(*b + *a);
}

void fiadd(Field3D*a,Field2D*b){
  *a += *b;
}

Field3D * fmul(Field3D*a,Field3D*b){
  Field3D * r=new Field3D(*a);
  *r *= *b;
  return r;
}

Field3D * fmul(Field3D*a,double b){
  Field3D * r=new Field3D(*a);
  *r *= b;
  return r;
}

Field3D * fmul(double b,Field3D*a){
  return new Field3D(b * *a);
}

void fimul(Field3D*a,Field3D*b){
  *a *= *b;
}

void fimul(Field3D*a,double b){
  *a *= b;
}

Field3D * fmul(Field3D*a,Field2D*b){
  Field3D * r=new Field3D(*a);
  *r *= *b;
  return r;
}

Field3D * fmul(Field2D*a,Field3D*b){
  return new Field3D(*b * *a);
}

void fimul(Field3D*a,Field2D*b){
  *a *= *b;
}

Field3D * ftruediv(Field3D*a,Field3D*b){
  Field3D * r=new Field3D(*a);
  *r /= *b;
  return r;
}

Field3D * ftruediv(Field3D*a,double b){
  Field3D * r=new Field3D(*a);
  *r /= b;
  return r;
}

Field3D * ftruediv(double b,Field3D*a){
  return new Field3D(b / *a);
}

void fitruediv(Field3D*a,Field3D*b){
  *a /= *b;
}

void fitruediv(Field3D*a,double b){
  *a /= b;
}

Field3D * ftruediv(Field3D*a,Field2D*b){
  Field3D * r=new Field3D(*a);
  *r /= *b;
  return r;
}

Field3D * ftruediv(Field2D*a,Field3D*b){
  return new Field3D(*b / *a);
}

void fitruediv(Field3D*a,Field2D*b){
  *a /= *b;
}

Field3D * fdiv(Field3D*a,Field3D*b){
  Field3D * r=new Field3D(*a);
  *r /= *b;
  return r;
}

Field3D * fdiv(Field3D*a,double b){
  Field3D * r=new Field3D(*a);
  *r /= b;
  return r;
}

Field3D * fdiv(double b,Field3D*a){
  return new Field3D(b / *a);
}

void fidiv(Field3D*a,Field3D*b){
  *a /= *b;
}

void fidiv(Field3D*a,double b){
  *a /= b;
}

Field3D * fdiv(Field3D*a,Field2D*b){
  Field3D * r=new Field3D(*a);
  *r /= *b;
  return r;
}

Field3D * fdiv(Field2D*a,Field3D*b){
  return new Field3D(*b / *a);
}

void fidiv(Field3D*a,Field2D*b){
  *a /= *b;
}

Field3D * fsub(Field3D*a,Field3D*b){
  Field3D * r=new Field3D(*a);
  *r -= *b;
  return r;
}

Field3D * fsub(Field3D*a,double b){
  Field3D * r=new Field3D(*a);
  *r -= b;
  return r;
}

Field3D * fsub(double b,Field3D*a){
  return new Field3D(b - *a);
}

void fisub(Field3D*a,Field3D*b){
  *a -= *b;
}

void fisub(Field3D*a,double b){
  *a -= b;
}

Field3D * fsub(Field3D*a,Field2D*b){
  Field3D * r=new Field3D(*a);
  *r -= *b;
  return r;
}

Field3D * fsub(Field2D*a,Field3D*b){
  return new Field3D(*b - *a);
}

void fisub(Field3D*a,Field2D*b){
  *a -= *b;
}

void c_set_f2d_all(Field2D * f, const double * data){
  int j=0;
  f->allocate();
  for (auto i : (*f)){
    (*f)[i]=data[j++];
  }
}

void c_set_f2d_all_(Field2D * f, const double data){
  (*f)=data;
}

void c_get_f2d_all(const Field2D * f, double * data){
  int j=0;
  for (auto i : (*f)){
    data[j++]=(*f)[i];
  }
}

int getNx( Field2D * a){
  return a->getNx();
}

int getNy( Field2D * a){
  return a->getNy();
}

int getNz( Field2D * a){
  return a->getNz();
}

Field2D * fadd(Field2D*a,Field2D*b){
  Field2D * r=new Field2D(*a);
  *r += *b;
  return r;
}

Field2D * fadd(Field2D*a,double b){
  Field2D * r=new Field2D(*a);
  *r += b;
  return r;
}

Field2D * fadd(double b,Field2D*a){
  return new Field2D(b + *a);
}

void fiadd(Field2D*a,Field2D*b){
  *a += *b;
}

void fiadd(Field2D*a,double b){
  *a += b;
}

Field2D * fmul(Field2D*a,Field2D*b){
  Field2D * r=new Field2D(*a);
  *r *= *b;
  return r;
}

Field2D * fmul(Field2D*a,double b){
  Field2D * r=new Field2D(*a);
  *r *= b;
  return r;
}

Field2D * fmul(double b,Field2D*a){
  return new Field2D(b * *a);
}

void fimul(Field2D*a,Field2D*b){
  *a *= *b;
}

void fimul(Field2D*a,double b){
  *a *= b;
}

Field2D * ftruediv(Field2D*a,Field2D*b){
  Field2D * r=new Field2D(*a);
  *r /= *b;
  return r;
}

Field2D * ftruediv(Field2D*a,double b){
  Field2D * r=new Field2D(*a);
  *r /= b;
  return r;
}

Field2D * ftruediv(double b,Field2D*a){
  return new Field2D(b / *a);
}

void fitruediv(Field2D*a,Field2D*b){
  *a /= *b;
}

void fitruediv(Field2D*a,double b){
  *a /= b;
}

Field2D * fdiv(Field2D*a,Field2D*b){
  Field2D * r=new Field2D(*a);
  *r /= *b;
  return r;
}

Field2D * fdiv(Field2D*a,double b){
  Field2D * r=new Field2D(*a);
  *r /= b;
  return r;
}

Field2D * fdiv(double b,Field2D*a){
  return new Field2D(b / *a);
}

void fidiv(Field2D*a,Field2D*b){
  *a /= *b;
}

void fidiv(Field2D*a,double b){
  *a /= b;
}

Field2D * fsub(Field2D*a,Field2D*b){
  Field2D * r=new Field2D(*a);
  *r -= *b;
  return r;
}

Field2D * fsub(Field2D*a,double b){
  Field2D * r=new Field2D(*a);
  *r -= b;
  return r;
}

Field2D * fsub(double b,Field2D*a){
  return new Field2D(b - *a);
}

void fisub(Field2D*a,Field2D*b){
  *a -= *b;
}

void fisub(Field2D*a,double b){
  *a -= b;
}

void c_get_f3d_part(const Field3D * f3d, double * data,int xs,int xe, int dx,int ys,int ye, int dy,int zs,int ze, int dz){
  int j=0;
  for (int x=xs;x!=xe;x+=dx){
    for (int y=ys;y!=ye;y+=dy){
      for (int z=zs;z!=ze;z+=dz){
        data[j++]=(*f3d)(x,y,z);
      }
    }
  }
}

void c_set_f3d_part(Field3D * f3d, const double * data,int xs,int xe, int dx,int ys,int ye, int dy,int zs,int ze, int dz){
  f3d->allocate();
  int j=0;
  for (int x=xs;x!=xe;x+=dx){
    for (int y=ys;y!=ye;y+=dy){
      for (int z=zs;z!=ze;z+=dz){
        (*f3d)(x,y,z)=data[j++];
      }
    }
  }
}
void c_set_f3d_part_(Field3D * f3d, const double data,int xs,int xe, int dx,int ys,int ye, int dy,int zs,int ze, int dz){
  f3d->allocate();
  for (int x=xs;x!=xe;x+=dx){
    for (int y=ys;y!=ye;y+=dy){
      for (int z=zs;z!=ze;z+=dz){
        (*f3d)(x,y,z)=data;
      }
    }
  }
}

void c_get_f2d_part(const Field2D * f2d, double * data,int xs,int xe, int dx,int ys,int ye, int dy){
  int j=0;
  for (int x=xs;x!=xe;x+=dx){
    for (int y=ys;y!=ye;y+=dy){
      data[j++]=(*f2d)(x,y);
    }
  }
}

void c_set_f2d_part(Field2D * f2d, const double * data,int xs,int xe, int dx,int ys,int ye, int dy){
  f2d->allocate();
  int j=0;
  for (int x=xs;x!=xe;x+=dx){
    for (int y=ys;y!=ye;y+=dy){
      (*f2d)(x,y)=data[j++];
    }
  }
}
void c_set_f2d_part_(Field2D * f2d, const double data,int xs,int xe, int dx,int ys,int ye, int dy){
  f2d->allocate();
  for (int x=xs;x!=xe;x+=dx){
    for (int y=ys;y!=ye;y+=dy){
      (*f2d)(x,y)=data;
    }
  }
}

Mesh * c_get_global_mesh(){
  return bout::globals::mesh;
}

void c_mesh_normalise(Mesh * msh, double norm){
  //printf("%g\n",norm);
  auto coord = msh->getCoordinates();
  coord->dx/=norm;
  coord->dy/=norm;
  coord->dz/=norm;
  coord->geometry();
}

